package main

import "fmt"

type DSU struct {
	dsuData []int
	mex []int
	excluded []bool
	size int
}

func NewDSU(size int) *DSU {
	dsu := DSU{
		dsuData: make([]int, size),
		mex: make([]int, size),
		excluded: make([]bool, size),
		size: size,
	}

	for i := range dsu.dsuData {
		dsu.dsuData[i] = -1
		dsu.mex[i] = -1
		dsu.excluded[i] = false
	}

	return &dsu
}

func (dsu *DSU) Find(vertex int) int {
	if dsu.dsuData[vertex] < 0 {
		return vertex
	}

	parent := dsu.Find(dsu.dsuData[vertex])
	dsu.dsuData[vertex] = parent

	return parent
}

func (dsu *DSU) Join(vertex1 int, vertex2 int) {
	set1 := dsu.Find(vertex1)
	set2 := dsu.Find(vertex2)

	if set1 == set2 {
		return
	}

	rank1 := -dsu.dsuData[set1]
	rank2 := -dsu.dsuData[set2]

	if rank1 < rank2 {
		set1, set2 = set2, set1
	}

	dsu.dsuData[set2] = set1

	if rank1 == rank2 {
		dsu.dsuData[set1] -= 1
	}

	if dsu.mex[set2] > dsu.mex[set1] {
		dsu.mex[set1] = dsu.mex[set2]
	}
}

func (dsu *DSU) GetMex() int {
	return dsu.mex[dsu.Find(0)]
}

func (dsu *DSU) MarkExcluded(vertex int) {
	if dsu.excluded[vertex] {
		return
	}

	dsu.excluded[vertex] = true
	dsu.mex[vertex] = vertex + 1

	if vertex < dsu.size - 1 && dsu.excluded[vertex + 1] {
		dsu.Join(vertex, vertex + 1);
	}

	if vertex > 0 && dsu.excluded[vertex - 1] {
		dsu.Join(vertex, vertex - 1);
	}
}

func main() {
	var length int
	
	fmt.Scanf("%d", &length)
	numbers1 := make([]int, length)
	numbers2 := make([]int, length)

	for i := 0; i < length; i++ {
		fmt.Scanf("%d", &numbers1[i])
	}

	for i := 0; i < length; i++ {
		fmt.Scanf("%d", &numbers2[i])
	}

	var queries int

	fmt.Scanf("%d", &queries)

	for i := 0; i < queries; i++ {
		var start, end int
		fmt.Scanf("%d %d", &start, &end)

		start -= 1

		dsu := NewDSU(length)

		for j := start; j < end; j++ {
			dsu.MarkExcluded(numbers1[j]);
		}

		fmt.Println(dsu.GetMex())
	}
}
