#include <iostream>
#include <vector>

using namespace std;

class DSU {
public:
    DSU(int size) {
        _parent = new int[size];
        _mex = new int[size];
        _excluded = new bool[size];
        _size = size;

        memset(_parent, 0xff, size * sizeof(int));
        memset(_mex, 0xff, size * sizeof(int));
        memset(_excluded, 0, size * sizeof(bool));
    }

    int find(int vertex) {
        if (_parent[vertex] < 0) {
            return vertex;
        }

        int result = find(_parent[vertex]);
        _parent[vertex] = result;

        return result;
    }

    void join(int vertex1, int vertex2) {
        vertex1 = find(vertex1);
        vertex2 = find(vertex2);

        if (vertex1 == vertex2) {
            return;
        }

        _parent[vertex2] = vertex1;
        _mex[vertex1] = max(_mex[vertex1], _mex[vertex2]);
    }

    void mark_excluded(int vertex) {
        if (_excluded[vertex]) {
            return;
        }

        _excluded[vertex] = true;
        _mex[vertex] = vertex + 1;

        if (vertex < _size - 1 && _excluded[vertex + 1]) {
            join(vertex, vertex + 1);
        }

        if (vertex > 0 && _excluded[vertex - 1]) {
            join(vertex, vertex - 1);
        }
    }

    int get_mex() {
        return _mex[find(0)];
    }
private:
    int* _parent;
    int* _mex;
    bool* _excluded;
    int _size;
};

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);

    int length;
    cin >> length;

    int* numbers1 = new int[length];
    int* numbers2 = new int[length];

    for (int i = 0; i < length; i++) {
        cin >> numbers1[i];
    }

    for (int i = 0; i < length; i++) {
        cin >> numbers2[i];
    }

    int queries;
    cin >> queries;

    for (int i = 0; i < queries; i++) {
        int start, end;
        cin >> start >> end;
        
        start -= 1;

        DSU dsu(length + 1);

        for (int j = start; j < end; j++) {
            dsu.mark_excluded(numbers1[j]);
        }

        cout << dsu.get_mex() << "\n";
    }

    return 0;
}
